/*-----------------------------------------------------------------------------
Copyright 2007 Milan Babuskov

This file is part of Vodovod

Vodovod is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Vodovod is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Vodovod in file COPYING; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------------*/
#include <string>
#include <sstream>
#include <libintl.h>
#define _(String) gettext (String)
#include "sutils.h"
#include "keys.h"
//-----------------------------------------------------------------------------
bool Keyboard::isKeyDown(SDLKey k)
{
    return keysM[k] > 6;    // 6 = repeat delay
}
//-----------------------------------------------------------------------------
void Keyboard::setKeyDown(SDLKey k, bool state)
{
    if (keysM.find(k) == keysM.end())
        keysM.insert(std::pair<SDLKey,char>(k, state?6:0));
    else
        keysM[k] = state?6:0;
}
//-----------------------------------------------------------------------------
SDLKey Keyboard::update(bool updateRepeated)
{
    SDL_Event event;
    while (SDL_PollEvent(&event))
    {
        if (event.type == SDL_KEYDOWN)
        {
            SDLKey k = event.key.keysym.sym;
            if (keysM.find(k) == keysM.end())
                keysM.insert(std::pair<SDLKey,char>(k, 1));
            else
                keysM[k] = 1;
            return k;
        }
        if (event.type == SDL_KEYUP)
        {
            keysM[event.key.keysym.sym] = 0;
            continue;
        }
        if (event.type == SDL_JOYBUTTONDOWN)
        {
            SDLKey k = (SDLKey)(504 + event.jbutton.which*50 + event.jbutton.button);
            if (keysM.find(k) == keysM.end())
                keysM.insert(std::pair<SDLKey,char>(k, 1));
            else
                keysM[k] = 1;
            return k;
        }
        if (event.type == SDL_JOYBUTTONUP)
        {
            SDLKey k = (SDLKey)(504 + event.jbutton.which*50 + event.jbutton.button);
            keysM[k] = 0;
            continue;
        }
        if (event.type == SDL_JOYAXISMOTION)
        {
            //printf("Value = %d, Axis = %d\n", event.jaxis.value, event.jaxis.axis);
            const int sensitivity = 12000;
            int isOn = event.jaxis.value < -sensitivity || event.jaxis.value > sensitivity;
            SDLKey which = (SDLKey)(500 + event.jaxis.axis * 2);
            SDLKey other = (SDLKey)(which+1);

            if (keysM.find(which) == keysM.end())
                keysM.insert(std::pair<SDLKey,char>(which, 0));
            else
                keysM[which] = 0;
            if (keysM.find(other) == keysM.end())
                keysM.insert(std::pair<SDLKey,char>(other, 0));
            else
                keysM[other] = 0;
            if (isOn)
            {
                SDLKey pressed = (event.jaxis.value > 0) ? other : which;
                keysM[pressed] = 1;
                return pressed;
            }
        }
        if (event.type == SDL_QUIT)
            throw Exiter(_("Close button of window clicked"));
    }

    // update already pressed keys
    if (updateRepeated)
        for (std::map<SDLKey,char>::iterator it = keysM.begin(); it != keysM.end(); ++it)
            if ((*it).second)
                (*it).second++;

    return SDLK_LAST;   // this is probably never used by SDL, so it denotes that key isn't pressed
}
//-----------------------------------------------------------------------------
// fill the names
KeyNames::KeyNames()
{
    keynames.insert(std::pair<int, std::string>(0,_("UNKNOWN")));
    keynames.insert(std::pair<int, std::string>(8,_("BACKSPACE")));
    keynames.insert(std::pair<int, std::string>(9,_("TAB")));
    keynames.insert(std::pair<int, std::string>(12,_("CLEAR")));
    keynames.insert(std::pair<int, std::string>(13,_("RETURN")));
    keynames.insert(std::pair<int, std::string>(19,_("PAUSE")));
    keynames.insert(std::pair<int, std::string>(27,_("ESCAPE")));
    keynames.insert(std::pair<int, std::string>(32,_("SPACE")));
    keynames.insert(std::pair<int, std::string>(33,_("EXCLAIM")));
    keynames.insert(std::pair<int, std::string>(34,_("QUOTEDBL")));
    keynames.insert(std::pair<int, std::string>(35,_("HASH")));
    keynames.insert(std::pair<int, std::string>(36,_("DOLLAR")));
    keynames.insert(std::pair<int, std::string>(38,_("AMPERSAND")));
    keynames.insert(std::pair<int, std::string>(39,_("QUOTE")));
    keynames.insert(std::pair<int, std::string>(40,_("LEFTPAREN")));
    keynames.insert(std::pair<int, std::string>(41,_("RIGHTPAREN")));
    keynames.insert(std::pair<int, std::string>(42,_("ASTERISK")));
    keynames.insert(std::pair<int, std::string>(43,_("PLUS")));
    keynames.insert(std::pair<int, std::string>(44,_("COMMA")));
    keynames.insert(std::pair<int, std::string>(45,_("MINUS")));
    keynames.insert(std::pair<int, std::string>(46,_("PERIOD")));
    keynames.insert(std::pair<int, std::string>(47,_("SLASH")));
    keynames.insert(std::pair<int, std::string>(48,_("0")));
    keynames.insert(std::pair<int, std::string>(49,_("1")));
    keynames.insert(std::pair<int, std::string>(50,_("2")));
    keynames.insert(std::pair<int, std::string>(51,_("3")));
    keynames.insert(std::pair<int, std::string>(52,_("4")));
    keynames.insert(std::pair<int, std::string>(53,_("5")));
    keynames.insert(std::pair<int, std::string>(54,_("6")));
    keynames.insert(std::pair<int, std::string>(55,_("7")));
    keynames.insert(std::pair<int, std::string>(56,_("8")));
    keynames.insert(std::pair<int, std::string>(57,_("9")));
    keynames.insert(std::pair<int, std::string>(58,_("COLON")));
    keynames.insert(std::pair<int, std::string>(59,_("SEMICOLON")));
    keynames.insert(std::pair<int, std::string>(60,_("LESS")));
    keynames.insert(std::pair<int, std::string>(61,_("EQUALS")));
    keynames.insert(std::pair<int, std::string>(62,_("GREATER")));
    keynames.insert(std::pair<int, std::string>(63,_("QUESTION")));
    keynames.insert(std::pair<int, std::string>(64,_("AT")));
    keynames.insert(std::pair<int, std::string>(91,_("LEFTBRACKET")));
    keynames.insert(std::pair<int, std::string>(92,_("BACKSLASH")));
    keynames.insert(std::pair<int, std::string>(93,_("RIGHTBRACKET")));
    keynames.insert(std::pair<int, std::string>(94,_("CARET")));
    keynames.insert(std::pair<int, std::string>(95,_("UNDERSCORE")));
    keynames.insert(std::pair<int, std::string>(96,_("BACKQUOTE")));
    keynames.insert(std::pair<int, std::string>(97,_("A")));
    keynames.insert(std::pair<int, std::string>(98,_("B")));
    keynames.insert(std::pair<int, std::string>(99,_("C")));
    keynames.insert(std::pair<int, std::string>(100,_("D")));
    keynames.insert(std::pair<int, std::string>(101,_("E")));
    keynames.insert(std::pair<int, std::string>(102,_("F")));
    keynames.insert(std::pair<int, std::string>(103,_("G")));
    keynames.insert(std::pair<int, std::string>(104,_("H")));
    keynames.insert(std::pair<int, std::string>(105,_("I")));
    keynames.insert(std::pair<int, std::string>(106,_("J")));
    keynames.insert(std::pair<int, std::string>(107,_("K")));
    keynames.insert(std::pair<int, std::string>(108,_("L")));
    keynames.insert(std::pair<int, std::string>(109,_("M")));
    keynames.insert(std::pair<int, std::string>(110,_("N")));
    keynames.insert(std::pair<int, std::string>(111,_("O")));
    keynames.insert(std::pair<int, std::string>(112,_("P")));
    keynames.insert(std::pair<int, std::string>(113,_("Q")));
    keynames.insert(std::pair<int, std::string>(114,_("R")));
    keynames.insert(std::pair<int, std::string>(115,_("S")));
    keynames.insert(std::pair<int, std::string>(116,_("T")));
    keynames.insert(std::pair<int, std::string>(117,_("U")));
    keynames.insert(std::pair<int, std::string>(118,_("V")));
    keynames.insert(std::pair<int, std::string>(119,_("W")));
    keynames.insert(std::pair<int, std::string>(120,_("X")));
    keynames.insert(std::pair<int, std::string>(121,_("Y")));
    keynames.insert(std::pair<int, std::string>(122,_("Z")));
    keynames.insert(std::pair<int, std::string>(127,_("DELETE")));
    keynames.insert(std::pair<int, std::string>(256,_("KEYPAD0")));
    keynames.insert(std::pair<int, std::string>(257,_("KEYPAD1")));
    keynames.insert(std::pair<int, std::string>(258,_("KEYPAD2")));
    keynames.insert(std::pair<int, std::string>(259,_("KEYPAD3")));
    keynames.insert(std::pair<int, std::string>(260,_("KEYPAD4")));
    keynames.insert(std::pair<int, std::string>(261,_("KEYPAD5")));
    keynames.insert(std::pair<int, std::string>(262,_("KEYPAD6")));
    keynames.insert(std::pair<int, std::string>(263,_("KEYPAD7")));
    keynames.insert(std::pair<int, std::string>(264,_("KEYPAD8")));
    keynames.insert(std::pair<int, std::string>(265,_("KEYPAD9")));
    keynames.insert(std::pair<int, std::string>(266,_("KEYPAD_PERIOD")));
    keynames.insert(std::pair<int, std::string>(267,_("KEYPAD_DIVIDE")));
    keynames.insert(std::pair<int, std::string>(268,_("KEYPAD_MULTIPLY")));
    keynames.insert(std::pair<int, std::string>(269,_("KEYPAD_MINUS")));
    keynames.insert(std::pair<int, std::string>(270,_("KEYPAD_PLUS")));
    keynames.insert(std::pair<int, std::string>(271,_("KEYPAD_ENTER")));
    keynames.insert(std::pair<int, std::string>(272,_("KEYPAD_EQUALS")));
    keynames.insert(std::pair<int, std::string>(273,_("UP")));
    keynames.insert(std::pair<int, std::string>(274,_("DOWN")));
    keynames.insert(std::pair<int, std::string>(275,_("RIGHT")));
    keynames.insert(std::pair<int, std::string>(276,_("LEFT")));
    keynames.insert(std::pair<int, std::string>(277,_("INSERT")));
    keynames.insert(std::pair<int, std::string>(278,_("HOME")));
    keynames.insert(std::pair<int, std::string>(279,_("END")));
    keynames.insert(std::pair<int, std::string>(280,_("PAGEUP")));
    keynames.insert(std::pair<int, std::string>(281,_("PAGEDOWN")));
    keynames.insert(std::pair<int, std::string>(282,_("F1")));
    keynames.insert(std::pair<int, std::string>(283,_("F2")));
    keynames.insert(std::pair<int, std::string>(284,_("F3")));
    keynames.insert(std::pair<int, std::string>(285,_("F4")));
    keynames.insert(std::pair<int, std::string>(286,_("F5")));
    keynames.insert(std::pair<int, std::string>(287,_("F6")));
    keynames.insert(std::pair<int, std::string>(288,_("F7")));
    keynames.insert(std::pair<int, std::string>(289,_("F8")));
    keynames.insert(std::pair<int, std::string>(290,_("F9")));
    keynames.insert(std::pair<int, std::string>(291,_("F10")));
    keynames.insert(std::pair<int, std::string>(292,_("F11")));
    keynames.insert(std::pair<int, std::string>(293,_("F12")));
    keynames.insert(std::pair<int, std::string>(294,_("F13")));
    keynames.insert(std::pair<int, std::string>(295,_("F14")));
    keynames.insert(std::pair<int, std::string>(296,_("F15")));
    keynames.insert(std::pair<int, std::string>(300,_("NUMLOCK")));
    keynames.insert(std::pair<int, std::string>(301,_("CAPSLOCK")));
    keynames.insert(std::pair<int, std::string>(302,_("SCROLLOCK")));
    keynames.insert(std::pair<int, std::string>(303,_("RSHIFT")));
    keynames.insert(std::pair<int, std::string>(304,_("LSHIFT")));
    keynames.insert(std::pair<int, std::string>(305,_("RCTRL")));
    keynames.insert(std::pair<int, std::string>(306,_("LCTRL")));
    keynames.insert(std::pair<int, std::string>(307,_("RALT")));
    keynames.insert(std::pair<int, std::string>(308,_("LALT")));
    keynames.insert(std::pair<int, std::string>(309,_("RMETA")));
    keynames.insert(std::pair<int, std::string>(310,_("LMETA")));
    keynames.insert(std::pair<int, std::string>(315,_("HELP")));
    keynames.insert(std::pair<int, std::string>(316,_("PRINT")));
    keynames.insert(std::pair<int, std::string>(317,_("SYSREQ")));
    keynames.insert(std::pair<int, std::string>(318,_("BREAK")));
    keynames.insert(std::pair<int, std::string>(319,_("MENU")));
}
//-----------------------------------------------------------------------------
std::string KeyNames::getKeyName(const int& key)
{
    if (key > 499)  // joystick stuff
    {
        std::string dirs[] = { _("LEFT"), _("RIGHT"), _("UP"), _("DOWN") };
        std::stringstream ss;
        ss << _("JOY ") << (key-450)/50 << " ";
        int p = key%50;
        if (p < 4)  // directions
            ss << dirs[p];
        else
            ss << _("BUTTON ") << (p-3);
        std::string retval = ss.str();
        return retval;
    }
    if (keynames.find(key) == keynames.end())
    {
        std::stringstream ss;
        ss << _("KEY ") << key;
        std::string retval = ss.str();
        return retval;
    }
    else
        return keynames[key];
}
//-----------------------------------------------------------------------------
KeyNames &KeyNames::keyNamesProvider()
{
    static KeyNames k;
    return k;
}
//-----------------------------------------------------------------------------
