/*-----------------------------------------------------------------------------
Njamfont.cpp

Simple font class that loads font resource (fixed width/height bitmap)
and provides functions to render it to screen


Copyright 2003 Milan Babuskov

This file is part of Njam (http://njam.sourceforge.net).

Njam is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Njam is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Njam in file COPYING; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------------*/
#include <stdio.h>
#include <string>
#include <libintl.h>
#define _(String) gettext (String)
#include "SDL.h"
#include "sutils.h"
#include "njamfont.h"
//---------------------------------------------------------------------------
NjamFont::NjamFont(const char *Filename, int CharWidth, int CharHeight,
    SDL_Color color)
{
    m_Font = NULL;
    m_Color = color;

    printf(_("Loading font..."));
    std::string path(DATADIR);
    if (Filename[0] != '/')
        path += '/';
    path += Filename;
    m_Font = TTF_OpenFont(path.c_str(), CharHeight);
    if (!m_Font)
    {
        printf(_("FAILED.\n"));
        fputs((const char *)TTF_GetError(), stdout);
        return;
    }
    printf(_("OK.\n"));

    m_CharWidth = CharWidth;
    m_CharHeight = CharHeight;
}
//---------------------------------------------------------------------------
void NjamFont::setAlpha(Uint8 value)
{
    /* no op */
}
//---------------------------------------------------------------------------
NjamFont::~NjamFont()
{
    TTF_CloseFont(m_Font);
}
//---------------------------------------------------------------------------
unsigned int NjamFont::GetCharWidth()
{
    return m_CharWidth;
}
//---------------------------------------------------------------------------
unsigned int NjamFont::GetCharHeight()
{
    return m_CharHeight;
}
//---------------------------------------------------------------------------
void NjamFont::GetTextSize(const char *Text, Uint16& w, Uint16& h)
{
    if (!m_Font)
        w = h = 0;
    else
    {
        int wt, ht;
        TTF_SizeUTF8(m_Font, Text, &wt, &ht);
        w = wt;
        h = ht;
    }
}
//---------------------------------------------------------------------------
// Writes text centered at screen
bool NjamFont::WriteTextCentered(SDL_Surface *Destination, int y, const char *Text)
{
    int w,h;
    TTF_SizeUTF8(m_Font, Text, &w, &h);
    int xpos = (Destination->w - w) / 2;
    return WriteTextXY(Destination, xpos, y, Text);
}
//---------------------------------------------------------------------------
// Uses CharWidth and Height to split screen into Columns and Rows
bool NjamFont::WriteTextColRow(SDL_Surface *Destination, int Col, int Row, const char *Text)
{
    return WriteTextXY(Destination, Col * m_CharWidth, Row * m_CharHeight, Text);
}
//---------------------------------------------------------------------------
// Outputs Text into Destionation surface at coordinate (x, y)
bool NjamFont::WriteTextXY(SDL_Surface *Destination, int x, int y, const char *Text)
{
    if (!m_Font || !Destination)
        return false;

    int w,h;
    TTF_SizeUTF8(m_Font, Text, &w, &h);

    SDL_Surface *stemp = TTF_RenderUTF8_Blended(m_Font, Text, m_Color);
    if (!stemp)
    {
        puts(_("Failed to create text image."));
        fputs((const char *)TTF_GetError(), stdout);
        return false;
    }
    SDL_Rect dest;
    NjamSetRect(dest, x, y);

    if (0 != SDL_BlitSurface(stemp, 0, Destination, &dest))
    {
        puts(_("Failed to blit font character image."));
        fputs((const char *)SDL_GetError(), stdout);
        SDL_FreeSurface(stemp);
        return false;
    }
    SDL_FreeSurface(stemp);
    return true;
}
//---------------------------------------------------------------------------

